/*
 * GPIO access for tasks.
 *
 * Copyright (C) 2016 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "board.h"
#include "mips.h"
#include "cpu.h"
#include "paging.h"
#include "memory.h"
#include "task_gpio.h"

void task_gpio_init(unsigned short task)
{
	u32 virtual, physical;

        /* Map the I/O region to the task. */

	for (virtual = TASK_GPIO_BASE, physical = GPIO_BASE;
		virtual < (u32) TASK_GPIO_BASE + (u32) GPIO_REGION_SIZE;
		virtual += page_size(STAGE2_PAGESIZE), physical += page_size(STAGE2_PAGESIZE))
	{
		init_page_table(STAGE2_PAGE_TABLE_BASE, virtual, physical,
			page_size(STAGE2_PAGESIZE), TLB_WRITE, task);
	}
}

inline void task_gpio_set_pin(unsigned short pin)
{
        TASK_REG_GPIO_PXDATS(pin / 32) |= (1 << (pin % 32));
}

inline void task_gpio_clear_pin(unsigned short pin)
{
        TASK_REG_GPIO_PXDATS(pin / 32) &= ~(1 << (pin % 32));
}

inline int task_gpio_get_pin(unsigned short pin)
{
	return TASK_REG_GPIO_PXPIN(pin / 32) & (1 << (pin % 32));
}
